using System;
using System.Collections.Generic;
using System.Text;
using Office = Microsoft.Office.Core;
using System.Diagnostics;

namespace VSTO_CSOutlookTemplate
{
    class CommandBarsMaker
    {
        private object missing = System.Reflection.Missing.Value;

        /// <summary>
        /// Create a new CommandBar (menu/toolbar)
        /// </summary>
        /// <param name="Bars"></param>
        /// <param name="barName"></param>
        /// <param name="Show"></param>
        /// <param name="Enabled"></param>
        /// <returns></returns>
        public Office.CommandBar NewBar(Office.CommandBars Bars, string barName,
            bool Show, bool Enabled)
        {
            Office.CommandBar _bar = null;

            try
            {
                int j = 1;
                for (int i = Bars.Count; i >= j; i--)
                {
                    _bar = Bars[i];
                    if (_bar.Name == barName)
                    {
                        _bar.Delete();

                        break;
                    }
                }
                _bar = null;
                _bar = (Office.CommandBar)Bars.Add(barName, missing, missing, true);

                if (_bar != null)
                {
                    _bar.Enabled = Enabled;
                    _bar.Visible = Show;
                }
            }
            catch
            {
                _bar = null;
            }
            return _bar;
        }

        /// <summary>
        /// Create a new CommandBarPopup (pop-up menu)
        /// </summary>
        /// <param name="Bar"></param>
        /// <param name="popName"></param>
        /// <param name="Tag"></param>
        /// <param name="Show"></param>
        /// <param name="ProgID"></param>
        /// <param name="ShowDivider"></param>
        /// <param name="TipText"></param>
        /// <param name="Enabled"></param>
        /// <returns></returns>
        public Office.CommandBarPopup NewMenu(Office.CommandBar Bar, string popName,
            object Tag, bool Show, string ProgID, bool ShowDivider, string TipText,
            bool Enabled)
        {
            Office.CommandBarPopup _pop = null;

            try
            {
                _pop = (Office.CommandBarPopup)Bar.FindControl(Microsoft.Office.Core.MsoControlType.msoControlPopup, missing, Tag, missing, true);

                if (_pop != null)
                {
                    _pop.Delete(false);
                    _pop = null;
                }

                _pop = (Office.CommandBarPopup)Bar.Controls.Add(Office.MsoControlType.msoControlPopup, missing, missing, missing, true);

                if (_pop != null)
                {
                    _pop.Tag = Tag.ToString();
                    _pop.Caption = popName;
                    _pop.Visible = Show;
                    _pop.Enabled = Enabled;
                    _pop.BeginGroup = ShowDivider;
                    _pop.OnAction = ProgID;
                    _pop.TooltipText = TipText;
                }
            }
            catch
            {
                _pop = null;
            }
            return _pop;
        }

        /// <summary>
        /// Create a new Button and add it to the specified CommandBar (menu/toolbar)
        /// </summary>
        /// <param name="Bar"></param>
        /// <param name="buttonName"></param>
        /// <param name="Tag"></param>
        /// <param name="Show"></param>
        /// <param name="ProgID"></param>
        /// <param name="Style"></param>
        /// <param name="ShowDivider"></param>
        /// <param name="TipText"></param>
        /// <param name="Enabled"></param>
        /// <returns></returns>
        public Office.CommandBarButton NewButton(Office.CommandBar Bar, 
            string buttonName, string Tag, bool Show, string ProgID, 
            Office.MsoButtonStyle Style, bool ShowDivider,
            string TipText, bool Enabled)
        {
            Office.CommandBarButton _button = null;

            try
            {
                _button = (Office.CommandBarButton)Bar.FindControl(missing, missing, Tag, missing, true);

                if (_button != null)
                {
                    _button.Delete(false);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }

            object buttonType = Office.MsoControlType.msoControlButton;

            _button = (Office.CommandBarButton)Bar.Controls.Add(buttonType, missing, missing, missing, true);

            if (_button != null)
            {
                _button.Caption = buttonName;
                _button.Visible = Show;
                _button.Enabled = Enabled;
                _button.Style = Style;
                _button.BeginGroup = ShowDivider;
                _button.Tag = Tag;
                _button.OnAction = ProgID;
                _button.TooltipText = TipText;
            }
            return _button;
        }

    /// <summary>
        /// Create a new ComboBox and add it to the specified CommandBar (menu/toolbar)
        /// </summary>
        /// <param name="Bar"></param>
        /// <param name="comboName"></param>
        /// <param name="Tag"></param>
        /// <param name="Show"></param>
        /// <param name="ProgID"></param>
        /// <param name="Style"></param>
        /// <param name="ShowDivider"></param>
        /// <param name="TipText"></param>
        /// <param name="Enabled"></param>
        /// <returns></returns>
        public Office.CommandBarComboBox NewComboBox(Office.CommandBar Bar, 
            string comboName, string Tag, bool Show, string ProgID,
            Office.MsoComboStyle Style, bool ShowDivider,
            string TipText, bool Enabled)
        {
            Office.CommandBarComboBox _combo = null;

            try
            {
                _combo = (Office.CommandBarComboBox)Bar.FindControl(missing, missing, Tag, missing, true);

                if (_combo != null)
                {
                    _combo.Delete(false);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }

            object controlType = Office.MsoControlType.msoControlComboBox;

            _combo = (Office.CommandBarComboBox)Bar.Controls.Add(controlType, missing, missing, missing, true);

            if (_combo != null)
            {
                _combo.Caption = comboName;
                _combo.Visible = Show;
                _combo.Enabled = Enabled;
                _combo.Style = Style;
                _combo.BeginGroup = ShowDivider;
                _combo.Tag = Tag;
                _combo.OnAction = ProgID;
                _combo.TooltipText = TipText;
            }
            return _combo;
        }
    }
}
