namespace VSTO_CSOutlookTemplate
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.IO;
    using System.Text;
    using System.Reflection;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;
    using stdole;
    using System.Drawing;
    using Office = Microsoft.Office.Core;
    using Outlook = Microsoft.Office.Interop.Outlook;
    using Forms = Microsoft.Vbe.Interop.Forms;
    
    // change the name for your own addin
    using Resources = VSTO_CSOutlookTemplate.Properties.Resources;

    // TODO:
    // This is an override of the RequestService method in the ThisAddIn class.
    // To hook up your custom ribbon uncomment this code.
    public partial class ThisAddIn
    {
        private Ribbon1 ribbon;
        private TimeTravelFormRegionWrapper FormRegionWrapper;

        private CTPWrapper TaskPaneWrapper;

        protected override object RequestService(Guid serviceGuid)
        {
            if (serviceGuid == typeof(Office.IRibbonExtensibility).GUID)
            {
                if (ribbon == null)
                    ribbon = new Ribbon1();
                return ribbon;
            }
            if (serviceGuid == typeof(Outlook.FormRegionStartup).GUID)
            {
                if (FormRegionWrapper == null)
                    FormRegionWrapper = new TimeTravelFormRegionWrapper();
                return FormRegionWrapper;
            }
            if (serviceGuid == typeof(Office.ICustomTaskPaneConsumer).GUID)
            {
                if (TaskPaneWrapper == null)
                    TaskPaneWrapper = new CTPWrapper();
                return TaskPaneWrapper;
            }
            return base.RequestService(serviceGuid);
        }
    }

    [ComVisible(true)]
    public class Ribbon1 : Office.IRibbonExtensibility
    {
        //Ribbon UI object
        private Office.IRibbonUI ribbon;

        public Ribbon1()
        {
        }

        #region IRibbonExtensibility Members

        public string GetCustomUI(string ribbonID)
        {
            //RibbonID indicates type of Inspector that is about to be displayed,
            //   valid RibbonID values are as follows:
            //Microsoft.Outlook.Mail.Read
            //Microsoft.Outlook.Mail.Compose
            //Microsoft.Outlook.MeetingRequest.Read
            //Microsoft.Outlook.MeetingRequest.Send
            //Microsoft.Outlook.Appointment
            //Microsoft.Outlook.Contact
            //Microsoft.Outlook.Journal
            //Microsoft.Outlook.Task
            //Microsoft.Outlook.DistributionList
            //Microsoft.Outlook.Report
            //Microsoft.Outlook.Resend
            //Microsoft.Outlook.Response.Read
            //Microsoft.Outlook.Response.Compose
            //Microsoft.Outlook.Response.CounterPropose
            //Microsoft.Outlook.RSS
            //Microsoft.Outlook.Post.Read
            //Microsoft.Outlook.Post.Compose
            //Microsoft.Outlook.Sharing.Read
            //Microsoft.Outlook.Sharing.Compose

            //In this sample only new mail Inspector is handled for a button.

            switch (ribbonID)
            {
                case "Microsoft.Outlook.Mail.Compose":
                    // Return the RibbonX markup stored as a resource in the project
                    return Resources.CustomRibbon;
                default:
                    {
                        return String.Empty;
                    }
            }
        }

        #endregion

        #region "Ribbon Callbacks"
        public void CS_Action(Office.IRibbonControl control)
        {
            Debug.WriteLine("CS_Action " + control.Id.ToString());

            if (control.Id == "VSTO_CS6")
            {
                try
                {
                    Outlook.Inspector myInsp = (Outlook.Inspector)control.Context;
                    System.Collections.SortedList wrap = (System.Collections.SortedList)ThisAddIn.AddinObject.InspectorWrapper();

                    int j = wrap.Count;

                    for (int i = 0; i < j; i++)
                    {
                        InspWrap wrapped = (InspWrap)wrap.GetByIndex(i);

                        if (wrapped.Inspector == myInsp)
                        {
                            wrapped.TaskPane.Visible = true;
                        }
                        else
                        {
                            wrapped.TaskPane.Visible = false;
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        public void CSToggle(Office.IRibbonControl control, bool pressed)
        {
            Debug.WriteLine("CSToggle " + control.Id.ToString());
            // pressed is True if the new button state is On.

            try
            {
                Outlook.Inspector myInsp = (Outlook.Inspector)control.Context;
                System.Collections.SortedList wrap = (System.Collections.SortedList)ThisAddIn.AddinObject.InspectorWrapper();

                int j = wrap.Count;

                for (int i = 0; i < j; i++)
                {
                    InspWrap wrapped = (InspWrap)wrap.GetByIndex(i);

                    if (wrapped.Inspector == myInsp)
                    {
                        switch (control.Id)
                        {
                            case "VSTO_CS1":
                                wrapped.HTML = true;
                                wrapped.PT = false;
                                wrapped.RTF = false;
                                wrapped.MailItem.BodyFormat = Outlook.OlBodyFormat.olFormatHTML;
                                ribbon.InvalidateControl("VSTO_CS1");
                                ribbon.InvalidateControl("VSTO_CS2");
                                ribbon.InvalidateControl("VSTO_CS3");
                                break;
                            case "VSTO_CS2":
                                wrapped.HTML = false;
                                wrapped.PT = true;
                                wrapped.RTF = false;
                                wrapped.MailItem.BodyFormat = Outlook.OlBodyFormat.olFormatPlain;
                                ribbon.InvalidateControl("VSTO_CS1");
                                ribbon.InvalidateControl("VSTO_CS2");
                                ribbon.InvalidateControl("VSTO_CS3");
                                break;
                            case "VSTO_CS3":
                                wrapped.HTML = false;
                                wrapped.PT = false;
                                wrapped.RTF = true;
                                wrapped.MailItem.BodyFormat = Outlook.OlBodyFormat.olFormatRichText;
                                ribbon.InvalidateControl("VSTO_CS1");
                                ribbon.InvalidateControl("VSTO_CS2");
                                ribbon.InvalidateControl("VSTO_CS3");
                                break;
                            default:
                                {
                                    break;
                                }
                        }
                        break;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        public bool CSPressed(Office.IRibbonControl control)
        {
            Debug.WriteLine("CS_GetPressed " + control.Id.ToString());

            bool retVal = false;

            try
            {
                Outlook.Inspector myInsp = (Outlook.Inspector)control.Context;
                System.Collections.SortedList wrap = (System.Collections.SortedList)ThisAddIn.AddinObject.InspectorWrapper();

                int j = wrap.Count;

                for (int i = 0; i < j; i++)
                {
                    InspWrap wrapped = (InspWrap)wrap.GetByIndex(i);

                    if (wrapped.Inspector == myInsp)
                    {
                        switch (control.Id)
                        {
                            case "VSTO_CS1":
                                retVal= wrapped.HTML;
                                break;
                            case "VSTO_CS2":
                                retVal = wrapped.PT;
                                break;
                            case "VSTO_CS3":
                                retVal = wrapped.RTF;
                                break;
                            default:
                            {
                                retVal = false;
                                break;
                            }
                        }
                        break;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

            return retVal;
        }

        public stdole.IPictureDisp CS_GetImage(Office.IRibbonControl control)
        {
            System.Drawing.Image imgStreamPic;
            IPictureDisp oPic = null;
            MyAxHost ax = new MyAxHost();

            switch (control.Id)
            {
                case "VSTO_CS1":
                    imgStreamPic = Resources.BALLOON;
                    break;
                case "VSTO_CS2":
                    imgStreamPic = Resources.BELL;
                    break;
                case "VSTO_CS3":
                    imgStreamPic = Resources.CUP;
                    break;
                case "VSTO_CS4":
                    imgStreamPic = Resources.ENVELOPE;
                    break;
                case "VSTO_CS6":
                    imgStreamPic = Resources.KEY;
                    break;
                default:
                    {
                        imgStreamPic = Resources.HAPPY;
                        break;
                    }
            }

            oPic = ax.IPictureDispFromImage(imgStreamPic);

            imgStreamPic = null;
            ax = null;

            return oPic;
        }

        public bool CS_GetVisible(Office.IRibbonControl control)
        {
            Debug.WriteLine("Control_" + control.Id);
            return true;
        }

        public string CS_GetLabel(Office.IRibbonControl control)
        {
            string Name = "CS_GetLabel Control_" + control.Id;
            Debug.WriteLine(Name);
            return Name;
        }

        public string CS_GetSuperTip(Office.IRibbonControl control)
        {
            string Name = "Tip_" + control.Id;
            Debug.WriteLine(Name);
            return Name;
        }

        public void Ribbon_OnLoad(Office.IRibbonUI Ribbon)
        {
            Debug.WriteLine("Ribbon_OnLoad");
            this.ribbon = Ribbon;

            System.Collections.SortedList wrap = (System.Collections.SortedList)ThisAddIn.AddinObject.InspectorWrapper();
            int j = wrap.Count;

            for (int i = 0; i < j; i++)
            {
                InspWrap wrapped = (InspWrap)wrap.GetByIndex(i);

                if (wrapped.Ribbon == null)
                {
                    wrapped.Ribbon = this.ribbon;
                }
            }
        }
        #endregion

        #region Ribbon Properties
        public Office.IRibbonUI Ribbon
        {
            get
            {
                return this.ribbon;
            }
        }
        #endregion
    }

    [ComVisible(true)]
    public class TimeTravelFormRegionWrapper : Outlook.FormRegionStartup
    {
        // required for Form Region support
        //Implements Outlook.FormRegionStartup
        #region Class_Declarations
        // Form Region declarations

        private Outlook.Application m_Outlook = null;

        // use the value in the <name> tag in the Region XML
        private const string TIME_TRAVEL_FORM_REGION = "TimeTravel"; 
        private Outlook.FormRegion m_FormRegion;
        private Forms.UserForm m_Form;

        private Outlook.AppointmentItem oAppt;

        private Outlook.OlkDateControl TravelToStartDate;
        private Outlook.OlkTimeControl TravelToStartTime;
        private Outlook.OlkTimeZoneControl TravelToStartTimeZone;
        private Outlook.OlkDateControl TravelToEndDate;
        private Outlook.OlkTimeControl TravelToEndTime;
        private Outlook.OlkTimeZoneControl TravelToEndTimeZone;
        private Outlook.OlkCheckBox ChkBillableTo;

        private Outlook.OlkDateControl TravelFromStartDate;
        private Outlook.OlkTimeControl TravelFromStartTime;
        private Outlook.OlkTimeZoneControl TravelFromStartTimeZone;
        private Outlook.OlkDateControl TravelFromEndDate;
        private Outlook.OlkTimeControl TravelFromEndTime;
        private Outlook.OlkTimeZoneControl TravelFromEndTimeZone;
        private Outlook.OlkCheckBox ChkBillableFrom;
        #endregion

        #region Initialization
        public void Init(Outlook.FormRegion Region, Outlook.Application OL)
        {
            m_FormRegion = Region;
            // hook up Close and Expanded event handlers
            m_FormRegion.Close += new Outlook.FormRegionEvents_CloseEventHandler(m_FormRegion_Close);
            m_FormRegion.Expanded += new Outlook.FormRegionEvents_ExpandedEventHandler(m_FormRegion_Expanded);

            oAppt = (Outlook.AppointmentItem)Region.Item;

            m_Outlook = OL;

            m_Form = (Forms.UserForm)m_FormRegion.Form;

            InitializeControls();
        }

        private void InitializeControls()
        {
            Outlook.TimeZone oZone = null;

            try
            {
                    TravelToStartDate = (Outlook.OlkDateControl)m_Form.Controls.Item("olkTravelToStartDate");
                    TravelToStartTime = (Outlook.OlkTimeControl)m_Form.Controls.Item("olkTravelToStartTime");
                    TravelToStartTimeZone = (Outlook.OlkTimeZoneControl)m_Form.Controls.Item("olkTravelToStartTimeZone");
                    TravelToEndDate = (Outlook.OlkDateControl)m_Form.Controls.Item("olkTravelToEndDate");
                    TravelToEndTime = (Outlook.OlkTimeControl)m_Form.Controls.Item("olkTravelToEndTime");
                    TravelToEndTimeZone = (Outlook.OlkTimeZoneControl)m_Form.Controls.Item("olkTravelToEndTimeZone");
                    ChkBillableTo = (Outlook.OlkCheckBox)m_Form.Controls.Item("olkChkBillableTo");

                    TravelFromStartDate = (Outlook.OlkDateControl)m_Form.Controls.Item("olkTravelFromStartDate");
                    TravelFromStartTime = (Outlook.OlkTimeControl)m_Form.Controls.Item("olkTravelFromStartTime");
                    TravelFromStartTimeZone = (Outlook.OlkTimeZoneControl)m_Form.Controls.Item("olkTravelFromStartTimeZone");
                    TravelFromEndDate = (Outlook.OlkDateControl)m_Form.Controls.Item("olkTravelFromEndDate");
                    TravelFromEndTime = (Outlook.OlkTimeControl)m_Form.Controls.Item("olkTravelFromEndTime");
                    TravelFromEndTimeZone = (Outlook.OlkTimeZoneControl)m_Form.Controls.Item("olkTravelFromEndTimeZone");
                    ChkBillableFrom = (Outlook.OlkCheckBox)m_Form.Controls.Item("olkChkBillableFrom");

                    oZone = m_Outlook.TimeZones.CurrentTimeZone;

                    TravelToStartDate.Value = DateTime.Today;
                    TravelToStartTime.Value = DateTime.Now;

                    TravelToStartTimeZone.AppointmentTimeField = Outlook.OlAppointmentTimeField.olAppointmentTimeFieldNone;
                    TravelToStartTimeZone.Value = oZone;

                    TravelToEndDate.Value = DateTime.Today;
                    TravelToEndTime.Value = DateTime.Now;

                    TravelToEndTimeZone.AppointmentTimeField = Outlook.OlAppointmentTimeField.olAppointmentTimeFieldNone;
                    TravelToEndTimeZone.Value = oZone;

                    ChkBillableTo.Value = true;

                    TravelFromStartDate.Value = DateTime.Today;
                    TravelFromStartTime.Value = DateTime.Now;

                    TravelFromStartTimeZone.AppointmentTimeField = Outlook.OlAppointmentTimeField.olAppointmentTimeFieldNone;
                    TravelFromStartTimeZone.Value = oZone;

                    TravelFromEndDate.Value = DateTime.Today;
                    TravelFromEndTime.Value = DateTime.Now;

                    TravelFromEndTimeZone.AppointmentTimeField = Outlook.OlAppointmentTimeField.olAppointmentTimeFieldNone;
                    TravelFromEndTimeZone.Value = oZone;

                    ChkBillableFrom.Value = true;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }

            oZone = null;
        }
        #endregion

        #region Form_Region_Support

        public object GetFormRegionIcon(string FormRegionName, int LCID, Outlook.OlFormRegionIcon Icon)
        {
            return null;
        }

        public object GetFormRegionManifest(string FormRegionName, int LCID)
        {
            return "";
        }

        public object GetFormRegionStorage(string FormRegionName, object Item, int LCID, Outlook.OlFormRegionMode FormRegionMode, Outlook.OlFormRegionSize FormRegionSize)
        {
            //Outlook.FormRegionStartup.GetFormRegionStorage

            // FormRegionName is the name of the region being loaded, can be returned as an OFS file as a Resource or as an OFS file.
            // Item is the Outlook item looking for the form region. Based on the registry entry for that MessageClass.
            // LCID is the locale setting for internationalization usage
            // FormRegionMode is either read, compose or preview
            // FormRegionSize is either adjoining or separate
            try
            {
                if (FormRegionName == TIME_TRAVEL_FORM_REGION)
                {
                    // return the name of the Resources file that's the Form Region OFS file
                    return Resources.TimeTravel;
                }
                else
                {
                    // if it's not for us return null
                    return null;
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                return null;
            }
        }

        public void BeforeFormRegionShow(Outlook.FormRegion FormRegion)
        {
            //Implements Outlook.FormRegionStartup.BeforeFormRegionShow
            
            m_FormRegion = FormRegion;

            System.Collections.SortedList wrap = (System.Collections.SortedList)ThisAddIn.AddinObject.InspectorWrapper();

            int j = wrap.Count;

            for (int i = 0; i < j; i++)
            {
                InspWrap wrapped = (InspWrap)wrap.GetByIndex(i);

                if (wrapped.Inspector == FormRegion.Inspector)
                {
                    wrapped.Region.Init(FormRegion, ThisAddIn.AddinObject.App());
                    break;
                }
            }
            wrap = null;
        }
        #endregion

        #region Form_Region_Events
        void m_FormRegion_Expanded(bool Expand)
        {
            // handles m_Region.Expanded
            Debug.WriteLine("Form Region expanded");
        }

        private void m_FormRegion_Close() 
        {
            //Handles m_FormRegion.Close

        }
        #endregion
    }

    [ComVisible(true)]
    public class CTPWrapper : Office.ICustomTaskPaneConsumer
    {
        // required for CTP use
        //Implements Office.ICustomTaskPaneConsumer

        // Task Pane declaration
        public Office.ICTPFactory m_CTP;

        public void CTPFactoryAvailable(Office.ICTPFactory CTPFactoryInst)
        {
            //Implements Office.ICustomTaskPaneConsumer.CTPFactoryAvailable
            m_CTP = CTPFactoryInst;
        }
    }
}