namespace VSTO_CSOutlookTemplate
{
    using System;
    using System.Windows.Forms;
    using Microsoft.VisualStudio.Tools.Applications.Runtime;
    using System.Reflection;
    using System.Collections.Generic;
    using System.Text;
    using System.Diagnostics;
    using System.Runtime.InteropServices;
    using Outlook = Microsoft.Office.Interop.Outlook;
    using Office = Microsoft.Office.Core;
    using Microsoft.Office.Tools;

    // When using this template make sure to change the AssemblyInfo module to use your own 
    //   settings and change the addin GUID to a new unique GUID, as well as changing the 
    //   ProgId for the addin.
    //
    // Also change the target folder for deployment in the Setup project.
    //
    //    
    // This template doesn't include the SetSecurity project needed for setting CAS
    //   security so the VSTO addin can load in Outlook. Download the SetSecurity 
    //   project and the VSTO 2005 SE deployment walkthrough from the MSDN Web
    //   site and use that material to successfully deploy a VSTO addin.

    public partial class ThisAddIn
    {
        #region Module_level_declarations

        // start of COM objects
        private Outlook.Application m_Outlook = null;

        private Outlook.NameSpace m_NameSpace = null;

        // Event-aware references to Explorers collection & ActiveExplorer
        private Outlook.Explorers m_Explorers = null;
        private Outlook.ExplorersClass m_ExplorersClass = null;
        private Outlook.Explorer m_Explorer = null;
        private Outlook.ExplorerClass m_ExplorerClass = null;

        // Event-aware references to Inspectors collection & ActiveInspector
        private Outlook.Inspectors m_InspectorClasss = null;
        private Outlook.InspectorsClass m_InspectorClasssClass = null;
        private Outlook.Inspector m_Inspector = null;
        private Outlook.InspectorClass m_InspectorClass = null;

        // end of COM objects

        // Explorer Wrapper Collection
        private System.Collections.SortedList m_ExplWrap = null;

        // Inspector Wrapper Collection
        private System.Collections.SortedList m_InspWrap = null;

        private string m_ProgID = "";

        private int m_WrapperID = 0;

        private int m_OutlookVersion = 0;

        //Initialization flags
        private bool m_Teardown = false;

        private bool m_Init = false;

        //Ribbon UI object
        private Office.IRibbonUI m_Ribbon;

        //CTP objects
        private Office.CustomTaskPane CTP;
        // change the strings for your own addin
        private const string CTP_PROGID = "VSTO_CSOutlookTemplate.EmailTaskPane";
        private const string CTP_EXPL_PROGID = "VSTO_CSOutlookTemplate.CustomTaskPane";
        private const string CTP_TITLE = "VSTO CS Config";
        private const string CTP_EXPL_TITLE = "VSTO CS WEB";

        // Form Region declarations
        // use the value in the <name> tag in the Region XML
        // change the strings for your own addin
        private const string TIME_TRAVEL_FORM_REGION = "TimeTravel";
        private TimeTravelFormRegionWrapper m_RegionWrap;

        #endregion

        #region StartupShutdown
        private void ThisAddin_Startup(object sender, System.EventArgs e)
        {
            // allot space initially for 15 open Explorers at a time
            m_ExplWrap = new System.Collections.SortedList(15);

            // allot space initially for 15 open Inspectors at a time
            m_InspWrap = new System.Collections.SortedList(15);

            try
            {
                // get the ProgID of the addin to use later with buttons/toolbars
                AssemblyName thisAssemblyName = null;
                thisAssemblyName = Assembly.GetExecutingAssembly().GetName();
                m_ProgID = thisAssemblyName.Name; //ProgID for the addin
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            
            try
            {
                AddinObject = new AutomationObject(this, ref m_InspWrap, m_ProgID);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

            m_Outlook = this.Application;

            try
            {
                m_NameSpace = m_Outlook.GetNamespace("MAPI");

                //event-aware reference to Explorers collection
                //use NewExplorer event to watch for UI creation
                m_Explorers = m_Outlook.Explorers;

                try
                {
                    m_Teardown = false;

                    //Are we starting with UI?
                    if (m_Explorers.Count > 0)
                    {
                        m_Explorer = m_Outlook.Explorers[1];

                        //we have UI - initialize base class
                        InitHandler();

                        if (m_Init == true)
                        {
                            // Explorer custom task pane
                            // uncomment to create a task pane for the main Explorer.
                            //
                            // You can move this code to the Explorer wrapper class,
                            //  that way you can have individualized task panes.
                            // To do so make sure the TaskPaneFactory object is available.
                            //
                            //try
                            //{
                            //    CTP = TaskPaneWrapper.m_CTP.CreateCTP(CTP_EXPL_PROGID, CTP_EXPL_TITLE, m_Explorer);
                            //    CTP.DockPositionRestrict = Office.MsoCTPDockPositionRestrict.msoCTPDockPositionRestrictNoChange;
                            //    CTP.DockPosition = Office.MsoCTPDockPosition.msoCTPDockPositionRight;
                            //    CTP.Visible = true;
                            //}
                            //catch (Exception ex)
                            //{
                            //    MessageBox.Show(ex.Message);
                            //}
                            //

                            OutExpl adder = new OutExpl();

                            m_WrapperID = adder.AddExpl(m_Explorer, m_WrapperID, ref m_ExplWrap);

                            adder = null;
                        }

                        m_InspectorClasss = m_Outlook.Inspectors;

                        m_Outlook.OptionsPagesAdd += new Microsoft.Office.Interop.Outlook.ApplicationEvents_11_OptionsPagesAddEventHandler(m_objOutlook_OptionsPagesAdd);
                        m_NameSpace.OptionsPagesAdd += new Microsoft.Office.Interop.Outlook.NameSpaceEvents_OptionsPagesAddEventHandler(m_NameSpace_OptionsPagesAdd);

                        m_Explorers.NewExplorer += new Microsoft.Office.Interop.Outlook.ExplorersEvents_NewExplorerEventHandler(m_Explorers_NewExplorer);

                        m_ExplorerClass = (Outlook.ExplorerClass)m_Explorer;
                        m_ExplorerClass.ExplorerEvents_Event_Close += new Microsoft.Office.Interop.Outlook.ExplorerEvents_CloseEventHandler(m_Explorer_Close);

                        m_InspectorClasss.NewInspector += new Microsoft.Office.Interop.Outlook.InspectorsEvents_NewInspectorEventHandler(m_InspectorClasss_NewInspector);
                    }
                    else
                    {
                        //do nothing
                        //monitor Explorers collection (in this module)
                        //if NewExplorer event is raised then we have UI
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                    TearDown();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                TearDown();
            }
        }

        private void ThisAddin_Shutdown(object sender, System.EventArgs e)
        {
            // This cleans up the Outlook objects and other public objects.
            TearDown();
        }
        #endregion

        #region utility_procedures
        public void InitHandler()
        {
            string sVerLeft2 = "";
            string sVersion = "";

            //********************************************************

            sVersion = m_Outlook.Version;
            sVerLeft2 = sVersion.Substring(0, 2);
            switch (sVerLeft2)
            {
                case "10":
                    m_OutlookVersion = 10;
                    break;
                case "11":
                    m_OutlookVersion = 11;
                    break;
                case "12":
                    m_OutlookVersion = 12;
                    break;
                default:
                    if ((sVerLeft2.Substring(0, 1)) == "9")
                    {
                        m_OutlookVersion = 9;
                    }
                    else
                    {
                        m_OutlookVersion = 0;
                    }
                    break;
            }

            //set initialization flag
            m_Init = true;
        }

        private void TearDown()
        {
            if (m_Teardown == false)
            {
                try
                {
                    if (m_ExplWrap != null)
                    {
                        m_ExplWrap.Clear();
                        m_ExplWrap = null;
                    }

                    if (m_InspWrap != null)
                    {
                        m_InspWrap.Clear();
                        m_InspWrap = null;
                    }

                    // remove the event handlers
                    if (m_Explorers != null)
                    {
                        m_Explorers.NewExplorer -= new Microsoft.Office.Interop.Outlook.ExplorersEvents_NewExplorerEventHandler(m_Explorers_NewExplorer);
                    }

                    if (m_InspectorClasss != null)
                    {
                        m_InspectorClasss.NewInspector -= new Microsoft.Office.Interop.Outlook.InspectorsEvents_NewInspectorEventHandler(m_InspectorClasss_NewInspector);
                    }

                    //release Ribbon reference
                    m_Ribbon = null;

                    //release CTP reference
                    CTP = null;

                    //release reference to Outlook objects
                    if (m_Explorer != null) m_Explorer = null;
                    if (m_Explorers != null) m_Explorers = null;
                    if (m_InspectorClasss != null) m_InspectorClasss = null;
                    if (m_Inspector != null) m_Inspector = null;

                    if (m_ExplorersClass != null) m_ExplorersClass = null;
                    if (m_ExplorerClass != null) m_ExplorerClass = null;

                    if (m_InspectorClasssClass != null) m_InspectorClasssClass = null;
                    if (m_InspectorClass != null) m_InspectorClass = null;

                    if (m_NameSpace != null) m_NameSpace = null;
                    if (m_Outlook != null) m_Outlook = null;

                    m_Teardown = true;
                    m_Init = false;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }
        #endregion

        #region explorer_related_events
        // NewExplorer event will be raised if there is UI
        private void m_Explorers_NewExplorer(Microsoft.Office.Interop.Outlook.Explorer Explorer)
        {
            //assign ActiveExplorer
            m_Explorer = Explorer;

            if (m_Init == false)
            {
                //we didn't have UI before - initialize add-in objects
                InitHandler();
            }

            if (m_Init == true)
            {
                OutExpl adder = new OutExpl();

                m_WrapperID = adder.AddExpl(Explorer, m_WrapperID, ref m_ExplWrap);

                adder = null;
            }
        }

        // Monitor Explorer_Close to see when UI "disappears"
        private void m_Explorer_Close()
        {
            //release current reference
            m_Explorer = null;
            try
            {
                m_Explorer = m_Outlook.ActiveExplorer();
            }
            catch
            {
                if (m_Outlook.Inspectors.Count == 0)
                {
                    //release add-in objects
                    if (m_Teardown == false) TearDown();
                }
            }
        }
        #endregion

        #region inspector_related_events
        private void m_InspectorClasss_NewInspector(Microsoft.Office.Interop.Outlook.Inspector Inspector)
        {
            //No handling of Inspectors for Notes, they are brain dead

            // set up to get the Class property of the item in the Inspector
            object item = Inspector.CurrentItem;

            Type _type;
            _type = item.GetType();

            object[] _args = new Object[] { };  // dummy argument array
            Outlook.OlObjectClass _class = Outlook.OlObjectClass.olNote;

            try // try to get the Class using reflection
            {
                _class = (Outlook.OlObjectClass)_type.InvokeMember("Class",
                    BindingFlags.Public | BindingFlags.GetField | BindingFlags.GetProperty,
                    null, item, _args);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                _class = Outlook.OlObjectClass.olNote;
            }

            if (_class != Outlook.OlObjectClass.olNote)
            {
                m_Inspector = Inspector;

                try
                {
                    m_InspectorClass = (Outlook.InspectorClass)m_Inspector;
                    m_InspectorClass.InspectorEvents_Event_Close += new Microsoft.Office.Interop.Outlook.InspectorEvents_CloseEventHandler(molInspector_Close);

                    OutInsp adder = new OutInsp();

                    switch (_class)
                    {
                        case Outlook.OlObjectClass.olMail:
                            // You can move this code to the Inspector wrapper class,
                            //  that way you can have individualized task panes.
                            // To do so make sure the TaskPaneFactory object is available.
                            CTP = TaskPaneWrapper.m_CTP.CreateCTP(CTP_PROGID, CTP_TITLE, Inspector);
                            CTP.DockPositionRestrict = Office.MsoCTPDockPositionRestrict.msoCTPDockPositionRestrictNoChange;
                            CTP.DockPosition = Office.MsoCTPDockPosition.msoCTPDockPositionRight;
                            CTP.Visible = false;
                            break;
                        case Outlook.OlObjectClass.olAppointment:
                            m_RegionWrap = new TimeTravelFormRegionWrapper();
                            break;
                        default:
                            {
                                break;
                            }
                    }

                    // m_RegionWrap and m_Ribbon are dummies, filled in when Ribbon_OnLoad and BeforeFormRegionShow fire
                    m_WrapperID = adder.AddInsp(Inspector, m_WrapperID, _class, ref m_InspWrap, CTP, m_RegionWrap, m_Ribbon);

                    adder = null;

                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void molInspector_Close()
        {
            m_Inspector = null;
            
            try
            {
                m_Inspector = m_Outlook.ActiveInspector();
            }
            catch
            {
                if (m_Inspector == null)
                {
                    if (m_Outlook.Explorers.Count == 0)
                    {
                        if (m_Teardown == false)
                        {
                            TearDown();
                        }
                    }
                }
            }
        }
        #endregion

        #region Property_Page_display_events
        private void m_NameSpace_OptionsPagesAdd(Microsoft.Office.Interop.Outlook.PropertyPages Pages, Microsoft.Office.Interop.Outlook.MAPIFolder Folder)
        {
            FolderPP oPP = new FolderPP();
            // change the strings for your own addin
            string caption = "VSTO C# Outlook " + Folder.Name + " Property Page";

            oPP.Caption = caption;
            Pages.Add(oPP, caption);

            oPP = null;
        }

        private void m_objOutlook_OptionsPagesAdd(Microsoft.Office.Interop.Outlook.PropertyPages Pages)
        {
            ToolsOptionsPP oPP = new ToolsOptionsPP();
            // change the strings for your own addin
            string caption = "VSTO C# Outlook Property Page";

            oPP.Caption = caption;
            Pages.Add(oPP, caption);

            oPP = null;
        }
        #endregion


        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddin_Startup);
            this.Shutdown += new System.EventHandler(ThisAddin_Shutdown);
        }

        #endregion

        public static AutomationObject AddinObject = null;
        private AddinUtilities addinUtilities;

        protected override object RequestComAddInAutomationService()
        {
            if (addinUtilities == null)
            {
                addinUtilities = new AddinUtilities();
            }
            return addinUtilities;
            //return base.RequestComAddInAutomationService();
        }
    }

    [ComVisible(true)]
    [InterfaceType(ComInterfaceType.InterfaceIsDual)]
    public interface IAddinUtilities
    {
        void CalledFromOutside();
    }

    [ComVisible(true)]
    [ClassInterface(ClassInterfaceType.None)]
    public class AddinUtilities : IAddinUtilities
    {
        // Demonstrates a method that can be called from outside the addin.
        // This technique can be used to call functions and to read/write properties.
        public void CalledFromOutside()
        {
            AutomationObject myAddinObject = ThisAddIn.AddinObject;
            string Count = myAddinObject.ExplCount();
            string ID = myAddinObject.ProgID();

            //MessageBox.Show("This is a test of an outside call to the COM addin");
            MessageBox.Show("There are currently " + Count + " Explorers open.");
        }
    }
}